@@ -1,7 +1,7 @@
 /*
  * -----------------------------------------------------------------
- * $Revision: 1.2 $
- * $Date: 2006/10/11 16:34:13 $
+ * $Revision: 1.7 $
+ * $Date: 2009/02/17 02:42:29 $
  * ----------------------------------------------------------------- 
  * Programmer(s): Scott D. Cohen, Alan C. Hindmarsh, Radu Serban,
  *                and Aaron Collier @ LLNL
@@ -22,6 +22,7 @@
 
 #include "cvode_impl.h"
 #include "cvode_bandpre_impl.h"
+#include "cvode_spils_impl.h"
 
 #include <cvode/cvode_sptfqmr.h>
 #include <cvode/cvode_spbcgs.h>
@@ -30,6 +31,7 @@
 #include <sundials/sundials_math.h>
 
 #define MIN_INC_MULT RCONST(1000.0)
+
 #define ZERO         RCONST(0.0)
 #define ONE          RCONST(1.0)
 
@@ -45,9 +47,13 @@ static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
                            realtype gamma, realtype delta,
                            int lr, void *bp_data, N_Vector tmp);
 
+/* Prototype for CVBandPrecFree */
+
+static void CVBandPrecFree(CVodeMem cv_mem);
+
 /* Prototype for difference quotient Jacobian calculation routine */
 
-static int CVBandPDQJac(CVBandPrecData pdata, 
+static int CVBandPDQJac(CVBandPrecData pdata,
                         realtype t, N_Vector y, N_Vector fy, 
                         N_Vector ftemp, N_Vector ytemp);
 
@@ -57,38 +63,46 @@ static int CVBandPDQJac(CVBandPrecData pdata,
 
 /*
  * -----------------------------------------------------------------
- * Malloc, Free, and Get Functions
+ * Initialization, Free, and Get Functions
  * NOTE: The band linear solver assumes a serial implementation
- *       of the NVECTOR package. Therefore, CVBandPrecAlloc will
+ *       of the NVECTOR package. Therefore, CVBandPrecInit will
  *       first test for a compatible N_Vector internal representation
  *       by checking that the function N_VGetArrayPointer exists.
  * -----------------------------------------------------------------
  */
 
-void *CVBandPrecAlloc(void *cvode_mem, long int N, 
-                      long int mu, long int ml)
+int CVBandPrecInit(void *cvode_mem, int N, int mu, int ml)
 {
   CVodeMem cv_mem;
+  CVSpilsMem cvspils_mem;
   CVBandPrecData pdata;
-  long int mup, mlp, storagemu;
+  int mup, mlp, storagemu;
+  int flag;
 
   if (cvode_mem == NULL) {
-    CVProcessError(NULL, 0, "CVBANDPRE", "CVBandPrecAlloc", MSGBP_CVMEM_NULL);
-    return(NULL);
+    CVProcessError(NULL, CVSPILS_MEM_NULL, "CVBANDPRE", "CVBandPrecInit", MSGBP_MEM_NULL);
+    return(CVSPILS_MEM_NULL);
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  /* Test if one of the SPILS linear solvers has been attached */
+  if (cv_mem->cv_lmem == NULL) {
+    CVProcessError(cv_mem, CVSPILS_LMEM_NULL, "CVBANDPRE", "CVBandPrecInit", MSGBP_LMEM_NULL);
+    return(CVSPILS_LMEM_NULL);
+  }
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+
   /* Test if the NVECTOR package is compatible with the BAND preconditioner */
   if(vec_tmpl->ops->nvgetarraypointer == NULL) {
-    CVProcessError(cv_mem, 0, "CVBANDPRE", "CVBandPrecAlloc", MSGBP_BAD_NVECTOR);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_ILL_INPUT, "CVBANDPRE", "CVBandPrecInit", MSGBP_BAD_NVECTOR);
+    return(CVSPILS_ILL_INPUT);
   }
 
   pdata = NULL;
   pdata = (CVBandPrecData) malloc(sizeof *pdata);  /* Allocate data memory */
   if (pdata == NULL) {
-    CVProcessError(cv_mem, 0, "CVBANDPRE", "CVBandPrecAlloc", MSGBP_MEM_FAIL);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBANDPRE", "CVBandPrecInit", MSGBP_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL);
   }
 
   /* Load pointers and bandwidths into pdata block. */
@@ -102,128 +116,72 @@ void *CVBandPrecAlloc(void *cvode_mem, long int N,
 
   /* Allocate memory for saved banded Jacobian approximation. */
   pdata->savedJ = NULL;
-  pdata->savedJ = BandAllocMat(N, mup, mlp, mup);
+  pdata->savedJ = NewBandMat(N, mup, mlp, mup);
   if (pdata->savedJ == NULL) {
     free(pdata); pdata = NULL;
-    CVProcessError(cv_mem, 0, "CVBANDPRE", "CVBandPrecAlloc", MSGBP_MEM_FAIL);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBANDPRE", "CVBandPrecInit", MSGBP_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL);
   }
 
   /* Allocate memory for banded preconditioner. */
   storagemu = MIN(N-1, mup+mlp);
   pdata->savedP = NULL;
-  pdata->savedP = BandAllocMat(N, mup, mlp, storagemu);
+  pdata->savedP = NewBandMat(N, mup, mlp, storagemu);
   if (pdata->savedP == NULL) {
-    BandFreeMat(pdata->savedJ);
+    DestroyMat(pdata->savedJ);
     free(pdata); pdata = NULL;
-    CVProcessError(cv_mem, 0, "CVBANDPRE", "CVBandPrecAlloc", MSGBP_MEM_FAIL);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBANDPRE", "CVBandPrecInit", MSGBP_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL);
   }
 
   /* Allocate memory for pivot array. */
   pdata->pivots = NULL;
-  pdata->pivots = BandAllocPiv(N);
+  pdata->pivots = NewIntArray(N);
   if (pdata->savedJ == NULL) {
-    BandFreeMat(pdata->savedP);
-    BandFreeMat(pdata->savedJ);
+    DestroyMat(pdata->savedP);
+    DestroyMat(pdata->savedJ);
     free(pdata); pdata = NULL;
-    CVProcessError(cv_mem, 0, "CVBANDPRE", "CVBandPrecAlloc", MSGBP_MEM_FAIL);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBANDPRE", "CVBandPrecInit", MSGBP_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL);
   }
 
-  return((void *) pdata);
+  /* Overwrite the P_data field in the SPILS memory */
+  cvspils_mem->s_P_data = pdata;
+
+  /* Attach the pfree function */
+  cvspils_mem->s_pfree = CVBandPrecFree;
+
+  /* Attach preconditioner solve and setup functions */
+  flag = CVSpilsSetPreconditioner(cvode_mem, CVBandPrecSetup, CVBandPrecSolve);
+
+  return(flag);
 }
 
-int CVBPSptfqmr(void *cvode_mem, int pretype, int maxl, void *p_data)
+int CVBandPrecGetWorkSpace(void *cvode_mem, long int *lenrwBP, long int *leniwBP)
 {
   CVodeMem cv_mem;
-  int flag;
+  CVSpilsMem cvspils_mem;
+  CVBandPrecData pdata;
+  int N, ml, mu, smu;
 
-  flag = CVSptfqmr(cvode_mem, pretype, maxl);
-  if(flag != CVSPILS_SUCCESS) return(flag);
   
-  cv_mem = (CVodeMem) cvode_mem;
-
-  if ( p_data == NULL ) {
-    CVProcessError(cv_mem, CVBANDPRE_PDATA_NULL, "CVBANDPRE", "CVBPSptfqmr", MSGBP_PDATA_NULL);
-    return(CVBANDPRE_PDATA_NULL);
-  } 
-
-  flag = CVSpilsSetPreconditioner(cvode_mem, CVBandPrecSetup, CVBandPrecSolve, p_data);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  return(CVSPILS_SUCCESS);
-}
-
-int CVBPSpbcg(void *cvode_mem, int pretype, int maxl, void *p_data)
-{
-  CVodeMem cv_mem;
-  int flag;
-
-  flag = CVSpbcg(cvode_mem, pretype, maxl);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  cv_mem = (CVodeMem) cvode_mem;
-
-  if ( p_data == NULL ) {
-    CVProcessError(cv_mem, CVBANDPRE_PDATA_NULL, "CVBANDPRE", "CVBPSpbcg", MSGBP_PDATA_NULL);
-    return(CVBANDPRE_PDATA_NULL);
-  } 
-
-  flag = CVSpilsSetPreconditioner(cvode_mem, CVBandPrecSetup, CVBandPrecSolve, p_data);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  return(CVSPILS_SUCCESS);
-}
-
-int CVBPSpgmr(void *cvode_mem, int pretype, int maxl, void *p_data)
-{
-  CVodeMem cv_mem;
-  int flag;
-
-  flag = CVSpgmr(cvode_mem, pretype, maxl);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  cv_mem = (CVodeMem) cvode_mem;
-
-  if ( p_data == NULL ) {
-    CVProcessError(cv_mem, CVBANDPRE_PDATA_NULL, "CVBANDPRE", "CVBPSpgmr", MSGBP_PDATA_NULL);
-    return(CVBANDPRE_PDATA_NULL);
+  if (cvode_mem == NULL) {
+    CVProcessError(NULL, CVSPILS_MEM_NULL, "CVBANDPRE", "CVBandPrecGetWorkSpace", MSGBP_MEM_NULL);
+    return(CVSPILS_MEM_NULL);
   }
+  cv_mem = (CVodeMem) cvode_mem;
 
-  flag = CVSpilsSetPreconditioner(cvode_mem, CVBandPrecSetup, CVBandPrecSolve, p_data);
-  if(flag != CVSPILS_SUCCESS) return(flag);
+  if (cv_mem->cv_lmem == NULL) {
+    CVProcessError(cv_mem, CVSPILS_LMEM_NULL, "CVBANDPRE", "CVBandPrecGetWorkSpace", MSGBP_LMEM_NULL);
+    return(CVSPILS_LMEM_NULL);
+  }
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
 
-  return(CVSPILS_SUCCESS);
-}
-
-void CVBandPrecFree(void **bp_data)
-{
-  CVBandPrecData pdata;
-
-  if (*bp_data == NULL) return;
-
-  pdata = (CVBandPrecData) (*bp_data);
-  BandFreeMat(pdata->savedJ);
-  BandFreeMat(pdata->savedP);
-  BandFreePiv(pdata->pivots);
-
-  free(*bp_data);
-  *bp_data = NULL;
-
-}
-
-int CVBandPrecGetWorkSpace(void *bp_data, long int *lenrwBP, long int *leniwBP)
-{
-  CVBandPrecData pdata;
-  long int N, ml, mu, smu;
-
-  if ( bp_data == NULL ) {
-    CVProcessError(NULL, CVBANDPRE_PDATA_NULL, "CVBANDPRE", "CVBandPrecGetWorkSpace", MSGBP_PDATA_NULL);
-    return(CVBANDPRE_PDATA_NULL);
+  if (cvspils_mem->s_P_data == NULL) {
+    CVProcessError(cv_mem, CVSPILS_PMEM_NULL, "CVBANDPRE", "CVBandPrecGetWorkSpace", MSGBP_PMEM_NULL);
+    return(CVSPILS_PMEM_NULL);
   } 
-
-  pdata = (CVBandPrecData) bp_data;
+  pdata = (CVBandPrecData) cvspils_mem->s_P_data;
 
   N   = pdata->N;
   mu  = pdata->mu;
@@ -233,52 +191,36 @@ int CVBandPrecGetWorkSpace(void *bp_data, long int *lenrwBP, long int *leniwBP)
   *leniwBP = pdata->N;
   *lenrwBP = N * ( 2*ml + smu + mu + 2 );
 
-  return(CVBANDPRE_SUCCESS);
+  return(CVSPILS_SUCCESS);
 }
 
-int CVBandPrecGetNumRhsEvals(void *bp_data, long int *nfevalsBP)
+int CVBandPrecGetNumRhsEvals(void *cvode_mem, long int *nfevalsBP)
 {
+  CVodeMem cv_mem;
+  CVSpilsMem cvspils_mem;
   CVBandPrecData pdata;
 
-  if (bp_data == NULL) {
-    CVProcessError(NULL, CVBANDPRE_PDATA_NULL, "CVBANDPRE", "CVBandPrecGetNumRhsEvals", MSGBP_PDATA_NULL);
-    return(CVBANDPRE_PDATA_NULL);
-  } 
+  if (cvode_mem == NULL) {
+    CVProcessError(NULL, CVSPILS_MEM_NULL, "CVBANDPRE", "CVBandPrecGetNumRhsEvals", MSGBP_MEM_NULL);
+    return(CVSPILS_MEM_NULL);
+  }
+  cv_mem = (CVodeMem) cvode_mem;
 
-  pdata = (CVBandPrecData) bp_data;
+  if (cv_mem->cv_lmem == NULL) {
+    CVProcessError(cv_mem, CVSPILS_LMEM_NULL, "CVBANDPRE", "CVBandPrecGetNumRhsEvals", MSGBP_LMEM_NULL);
+    return(CVSPILS_LMEM_NULL);
+  }
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+
+  if (cvspils_mem->s_P_data == NULL) {
+    CVProcessError(cv_mem, CVSPILS_PMEM_NULL, "CVBANDPRE", "CVBandPrecGetNumRhsEvals", MSGBP_PMEM_NULL);
+    return(CVSPILS_PMEM_NULL);
+  } 
+  pdata = (CVBandPrecData) cvspils_mem->s_P_data;
 
   *nfevalsBP = pdata->nfeBP;
 
-  return(CVBANDPRE_SUCCESS);
-}
-
-/*
- * -----------------------------------------------------------------
- * CVBandPrecGetReturnFlagName
- * -----------------------------------------------------------------
- */
-
-char *CVBandPrecGetReturnFlagName(int flag)
-{
-  char *name;
-
-  name = (char *)malloc(30*sizeof(char));
-
-  switch(flag) {
-  case CVBANDPRE_SUCCESS:
-    sprintf(name,"CVBANDPRE_SUCCESS");
-    break;   
-  case CVBANDPRE_PDATA_NULL:
-    sprintf(name,"CVBANDPRE_PDATA_NULL");
-    break;
-  case CVBANDPRE_RHSFUNC_UNRECVR:
-    sprintf(name,"CVBANDPRE_RHSFUNC_UNRECVR");
-    break;
-  default:
-    sprintf(name,"NONE");
-  }
-
-  return(name);
+  return(CVSPILS_SUCCESS);
 }
 
 /* Readability Replacements */
@@ -327,8 +269,7 @@ char *CVBandPrecGetReturnFlagName(int flag)
  *
  * gamma   is the scalar appearing in the Newton matrix.
  *
- * bp_data is a pointer to preconditoner data - the same as the
- *         bp_data parameter passed to CVSp*.
+ * bp_data is a pointer to preconditoner data (set by CVBandPrecInit)
  *
  * tmp1, tmp2, and tmp3 are pointers to memory allocated
  *           for vectors of length N for work space. This
@@ -345,7 +286,6 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
                            realtype gamma, void *bp_data,
                            N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
-  long int ier;
   CVBandPrecData pdata;
   CVodeMem cv_mem;
   int retval;
@@ -365,11 +305,11 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
 
     /* If jok = FALSE, call CVBandPDQJac for new J value. */
     *jcurPtr = TRUE;
-    BandZero(savedJ);
+    SetToZero(savedJ);
 
     retval = CVBandPDQJac(pdata, t, y, fy, tmp1, tmp2);
     if (retval < 0) {
-      CVProcessError(cv_mem, CVBANDPRE_RHSFUNC_UNRECVR, "CVBANDPRE", "CVBandPrecSetup", MSGBP_RHSFUNC_FAILED);
+      CVProcessError(cv_mem, -1, "CVBANDPRE", "CVBandPrecSetup", MSGBP_RHSFUNC_FAILED);
       return(-1);
     }
     if (retval > 0) {
@@ -382,13 +322,13 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
   
   /* Scale and add I to get savedP = I - gamma*J. */
   BandScale(-gamma, savedP);
-  BandAddI(savedP);
+  AddIdentity(savedP);
  
   /* Do LU factorization of matrix. */
-  ier = BandGBTRF(savedP, pivots);
+  retval = BandGBTRF(savedP, pivots);
  
   /* Return 0 if the LU was complete; otherwise return 1. */
-  if (ier > 0) return(1);
+  if (retval > 0) return(1);
   return(0);
 }
 
@@ -401,12 +341,11 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
  *
  * The parameters of CVBandPrecSolve used here are as follows:
  *
- * r       is the right-hand side vector of the linear system.
+ * r is the right-hand side vector of the linear system.
  *
- * bp_data is a pointer to preconditioner data - the same as the
- *         bp_data parameter passed to CVSp*.
+ * bp_data is a pointer to preconditoner data (set by CVBandPrecInit)
  *
- * z       is the output vector computed by CVBandPrecSolve.
+ * z is the output vector computed by CVBandPrecSolve.
  *
  * The value returned by the CVBandPrecSolve function is always 0,
  * indicating success.
@@ -435,11 +374,31 @@ static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
   return(0);
 }
 
-#define ewt    (cv_mem->cv_ewt)
-#define uround (cv_mem->cv_uround)
-#define h      (cv_mem->cv_h)
-#define f      (cv_mem->cv_f)
-#define f_data (cv_mem->cv_f_data)
+
+static void CVBandPrecFree(CVodeMem cv_mem)
+{
+  CVSpilsMem cvspils_mem;
+  CVBandPrecData pdata;
+
+  if (cv_mem->cv_lmem == NULL) return;
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+  
+  if (cvspils_mem->s_P_data == NULL) return;
+  pdata = (CVBandPrecData) cvspils_mem->s_P_data;
+
+  DestroyMat(savedJ);
+  DestroyMat(savedP);
+  DestroyArray(pivots);
+
+  free(pdata);
+  pdata = NULL;
+}
+
+#define ewt       (cv_mem->cv_ewt)
+#define uround    (cv_mem->cv_uround)
+#define h         (cv_mem->cv_h)
+#define f         (cv_mem->cv_f)
+#define user_data (cv_mem->cv_user_data)
 
 /*
  * -----------------------------------------------------------------
@@ -447,7 +406,7 @@ static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
  * -----------------------------------------------------------------
  * This routine generates a banded difference quotient approximation to
  * the Jacobian of f(t,y). It assumes that a band matrix of type
- * BandMat is stored column-wise, and that elements within each column
+ * DlsMat is stored column-wise, and that elements within each column
  * are contiguous. This makes it possible to get the address of a column
  * of J via the macro BAND_COL and to write a simple for loop to set
  * each of the elements of a column in succession.
@@ -460,7 +419,7 @@ static int CVBandPDQJac(CVBandPrecData pdata,
 {
   CVodeMem cv_mem;
   realtype fnorm, minInc, inc, inc_inv, srur;
-  long int group, i, j, width, ngroups, i1, i2;
+  int group, i, j, width, ngroups, i1, i2;
   realtype *col_j, *ewt_data, *fy_data, *ftemp_data, *y_data, *ytemp_data;
   int retval;
 
@@ -496,7 +455,7 @@ static int CVBandPDQJac(CVBandPrecData pdata,
 
     /* Evaluate f with incremented y. */
 
-    retval = f(t, ytemp, ftemp, f_data);
+    retval = f(t, ytemp, ftemp, user_data);
     nfeBP++;
     if (retval != 0) return(retval);
 
