@@ -1,7 +1,7 @@
 /*
  * -----------------------------------------------------------------
- * $Revision: 1.2 $
- * $Date: 2006/10/11 16:34:13 $
+ * $Revision: 1.8 $
+ * $Date: 2009/02/17 02:42:29 $
  * ----------------------------------------------------------------- 
  * Programmer(s): Michael Wittman, Alan C. Hindmarsh, Radu Serban,
  *                and Aaron Collier @ LLNL
@@ -21,16 +21,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "cvode_impl.h"
+#include "cvode_bbdpre_impl.h"
+#include "cvode_spils_impl.h"
+
 #include <cvode/cvode_sptfqmr.h>
 #include <cvode/cvode_spbcgs.h>
 #include <cvode/cvode_spgmr.h>
 
-#include "cvode_impl.h"
-#include "cvode_bbdpre_impl.h"
-
 #include <sundials/sundials_math.h>
 
 #define MIN_INC_MULT RCONST(1000.0)
+
 #define ZERO         RCONST(0.0)
 #define ONE          RCONST(1.0)
 
@@ -46,6 +48,10 @@ static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy,
                           realtype gamma, realtype delta,
                           int lr, void *bbd_data, N_Vector tmp);
 
+/* Prototype for CVBBDPrecFree */
+static void CVBBDPrecFree(CVodeMem cv_mem);
+
+
 /* Prototype for difference quotient Jacobian calculation routine */
 
 static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, 
@@ -59,38 +65,47 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t,
 
 /*
  * -----------------------------------------------------------------
- * User-Callable Functions: malloc, reinit and free
+ * User-Callable Functions: initialization, reinit and free
  * -----------------------------------------------------------------
  */
 
-void *CVBBDPrecAlloc(void *cvode_mem, long int Nlocal, 
-                     long int mudq, long int mldq,
-                     long int mukeep, long int mlkeep, 
-                     realtype dqrely, 
-                     CVLocalFn gloc, CVCommFn cfn)
+int CVBBDPrecInit(void *cvode_mem, int Nlocal, 
+                   int mudq, int mldq,
+                   int mukeep, int mlkeep, 
+                   realtype dqrely, 
+                   CVLocalFn gloc, CVCommFn cfn)
 {
   CVodeMem cv_mem;
+  CVSpilsMem cvspils_mem;
   CVBBDPrecData pdata;
-  long int muk, mlk, storage_mu;
+  int muk, mlk, storage_mu;
+  int flag;
 
   if (cvode_mem == NULL) {
-    CVProcessError(NULL, 0, "CVBBDPRE", "CVBBDPrecAlloc", MSGBBDP_CVMEM_NULL);
-    return(NULL);
+    CVProcessError(NULL, CVSPILS_MEM_NULL, "CVBBDPRE", "CVBBDPrecInit", MSGBBD_MEM_NULL);
+    return(CVSPILS_MEM_NULL);
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  /* Test if one of the SPILS linear solvers has been attached */
+  if (cv_mem->cv_lmem == NULL) {
+    CVProcessError(cv_mem, CVSPILS_LMEM_NULL, "CVBBDPRE", "CVBBDPrecInit", MSGBBD_LMEM_NULL);
+    return(CVSPILS_LMEM_NULL);
+  }
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+
   /* Test if the NVECTOR package is compatible with the BLOCK BAND preconditioner */
   if(vec_tmpl->ops->nvgetarraypointer == NULL) {
-    CVProcessError(cv_mem, 0, "CVBBDPRE", "CVBBDPrecAlloc", MSGBBDP_BAD_NVECTOR);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_ILL_INPUT, "CVBBDPRE", "CVBBDPrecInit", MSGBBD_BAD_NVECTOR);
+    return(CVSPILS_ILL_INPUT);
   }
 
   /* Allocate data memory */
   pdata = NULL;
   pdata = (CVBBDPrecData) malloc(sizeof *pdata);  
   if (pdata == NULL) {
-    CVProcessError(cv_mem, 0, "CVBBDPRE", "CVBBDPrecAlloc", MSGBBDP_MEM_FAIL);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBBDPRE", "CVBBDPrecInit", MSGBBD_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL);
   }
 
   /* Set pointers to gloc and cfn; load half-bandwidths */
@@ -105,32 +120,32 @@ void *CVBBDPrecAlloc(void *cvode_mem, long int Nlocal,
   pdata->mlkeep = mlk;
 
   /* Allocate memory for saved Jacobian */
-  pdata->savedJ = BandAllocMat(Nlocal, muk, mlk, muk);
+  pdata->savedJ = NewBandMat(Nlocal, muk, mlk, muk);
   if (pdata->savedJ == NULL) { 
     free(pdata); pdata = NULL; 
-    CVProcessError(cv_mem, 0, "CVBBDPRE", "CVBBDPrecAlloc", MSGBBDP_MEM_FAIL);
-    return(NULL); 
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBBDPRE", "CVBBDPrecInit", MSGBBD_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL); 
   }
 
   /* Allocate memory for preconditioner matrix */
   storage_mu = MIN(Nlocal-1, muk + mlk);
   pdata->savedP = NULL;
-  pdata->savedP = BandAllocMat(Nlocal, muk, mlk, storage_mu);
+  pdata->savedP = NewBandMat(Nlocal, muk, mlk, storage_mu);
   if (pdata->savedP == NULL) {
-    BandFreeMat(pdata->savedJ);
+    DestroyMat(pdata->savedJ);
     free(pdata); pdata = NULL;
-    CVProcessError(cv_mem, 0, "CVBBDPRE", "CVBBDPrecAlloc", MSGBBDP_MEM_FAIL);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBBDPRE", "CVBBDPrecInit", MSGBBD_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL);
   }
   /* Allocate memory for pivots */
   pdata->pivots = NULL;
-  pdata->pivots = BandAllocPiv(Nlocal);
+  pdata->pivots = NewIntArray(Nlocal);
   if (pdata->savedJ == NULL) {
-    BandFreeMat(pdata->savedP);
-    BandFreeMat(pdata->savedJ);
+    DestroyMat(pdata->savedP);
+    DestroyMat(pdata->savedJ);
     free(pdata); pdata = NULL;
-    CVProcessError(cv_mem, 0, "CVBBDPRE", "CVBBDPrecAlloc", MSGBBDP_MEM_FAIL);
-    return(NULL);
+    CVProcessError(cv_mem, CVSPILS_MEM_FAIL, "CVBBDPRE", "CVBBDPrecInit", MSGBBD_MEM_FAIL);
+    return(CVSPILS_MEM_FAIL);
   }
 
   /* Set pdata->dqrely based on input dqrely (0 implies default). */
@@ -144,92 +159,49 @@ void *CVBBDPrecAlloc(void *cvode_mem, long int Nlocal,
   pdata->ipwsize = Nlocal;
   pdata->nge = 0;
 
-  return((void *)pdata);
+  /* Overwrite the P_data field in the SPILS memory */
+  cvspils_mem->s_P_data = pdata;
+
+  /* Attach the pfree function */
+  cvspils_mem->s_pfree = CVBBDPrecFree;
+
+  /* Attach preconditioner solve and setup functions */
+  flag = CVSpilsSetPreconditioner(cvode_mem, CVBBDPrecSetup, CVBBDPrecSolve);
+
+  return(flag);
 }
 
-int CVBBDSptfqmr(void *cvode_mem, int pretype, int maxl, void *bbd_data)
+
+int CVBBDPrecReInit(void *cvode_mem, 
+                    int mudq, int mldq, 
+                    realtype dqrely)
 {
   CVodeMem cv_mem;
-  int flag;
-
-  flag = CVSptfqmr(cvode_mem, pretype, maxl);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  cv_mem = (CVodeMem) cvode_mem;
-
-  if (bbd_data == NULL) {
-    CVProcessError(cv_mem, CVBBDPRE_PDATA_NULL, "CVBBDPRE", "CVBBDSptfqmr", MSGBBDP_PDATA_NULL);
-    return(CVBBDPRE_PDATA_NULL);
-  } 
-
-  flag = CVSpilsSetPreconditioner(cvode_mem, CVBBDPrecSetup, CVBBDPrecSolve, bbd_data);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  return(CVSPILS_SUCCESS);
-}
-
-int CVBBDSpbcg(void *cvode_mem, int pretype, int maxl, void *bbd_data)
-{
-  CVodeMem cv_mem;
-  int flag;
-
-  flag = CVSpbcg(cvode_mem, pretype, maxl);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  cv_mem = (CVodeMem) cvode_mem;
-
-  if (bbd_data == NULL) {
-    CVProcessError(cv_mem, CVBBDPRE_PDATA_NULL, "CVBBDPRE", "CVBBDSpbcg", MSGBBDP_PDATA_NULL);
-    return(CVBBDPRE_PDATA_NULL);
-  } 
-
-  flag = CVSpilsSetPreconditioner(cvode_mem, CVBBDPrecSetup, CVBBDPrecSolve, bbd_data);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  return(CVSPILS_SUCCESS);
-}
-
-int CVBBDSpgmr(void *cvode_mem, int pretype, int maxl, void *bbd_data)
-{
-  CVodeMem cv_mem;
-  int flag;
-
-  flag = CVSpgmr(cvode_mem, pretype, maxl);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  cv_mem = (CVodeMem) cvode_mem;
-
-  if (bbd_data == NULL) {
-    CVProcessError(cv_mem, CVBBDPRE_PDATA_NULL, "CVBBDPRE", "CVBBDSpgmr", MSGBBDP_PDATA_NULL);
-    return(CVBBDPRE_PDATA_NULL);
-  } 
-
-  flag = CVSpilsSetPreconditioner(cvode_mem, CVBBDPrecSetup, CVBBDPrecSolve, bbd_data);
-  if(flag != CVSPILS_SUCCESS) return(flag);
-
-  return(CVSPILS_SUCCESS);
-}
-
-int CVBBDPrecReInit(void *bbd_data, 
-                    long int mudq, long int mldq, 
-                    realtype dqrely, 
-                    CVLocalFn gloc, CVCommFn cfn)
-{
+  CVSpilsMem cvspils_mem;
   CVBBDPrecData pdata;
-  CVodeMem cv_mem;
-  long int Nlocal;
+  int Nlocal;
 
-  if (bbd_data == NULL) {
-    CVProcessError(NULL, CVBBDPRE_PDATA_NULL, "CVBBDPRE", "CVBBDPrecReInit", MSGBBDP_PDATA_NULL);
-    return(CVBBDPRE_PDATA_NULL);
+  if (cvode_mem == NULL) {
+    CVProcessError(NULL, CVSPILS_MEM_NULL, "CVBBDPRE", "CVBBDPrecReInit", MSGBBD_MEM_NULL);
+    return(CVSPILS_MEM_NULL);
+  }
+  cv_mem = (CVodeMem) cvode_mem;
+
+  /* Test if one of the SPILS linear solvers has been attached */
+  if (cv_mem->cv_lmem == NULL) {
+    CVProcessError(cv_mem, CVSPILS_LMEM_NULL, "CVBBDPRE", "CVBBDPrecReInit", MSGBBD_LMEM_NULL);
+    return(CVSPILS_LMEM_NULL);
+  }
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+
+  /* Test if the preconditioner data is non-NULL */
+  if (cvspils_mem->s_P_data == NULL) {
+    CVProcessError(cv_mem, CVSPILS_PMEM_NULL, "CVBBDPRE", "CVBBDPrecReInit", MSGBBD_PMEM_NULL);
+    return(CVSPILS_PMEM_NULL);
   } 
+  pdata = (CVBBDPrecData) cvspils_mem->s_P_data;
 
-  pdata  = (CVBBDPrecData) bbd_data;
-  cv_mem = (CVodeMem) pdata->cvode_mem;
-
-  /* Set pointers to gloc and cfn; load half-bandwidths */
-  pdata->gloc = gloc;
-  pdata->cfn = cfn;
+  /* Load half-bandwidths */
   Nlocal = pdata->n_local;
   pdata->mudq = MIN(Nlocal-1, MAX(0,mudq));
   pdata->mldq = MIN(Nlocal-1, MAX(0,mldq));
@@ -240,85 +212,66 @@ int CVBBDPrecReInit(void *bbd_data,
   /* Re-initialize nge */
   pdata->nge = 0;
 
-  return(CVBBDPRE_SUCCESS);
+  return(CVSPILS_SUCCESS);
 }
 
-void CVBBDPrecFree(void **bbd_data)
-{
-  CVBBDPrecData pdata;
-  
-  if (*bbd_data == NULL) return;
-
-  pdata = (CVBBDPrecData) (*bbd_data);
-  BandFreeMat(pdata->savedJ);
-  BandFreeMat(pdata->savedP);
-  BandFreePiv(pdata->pivots);
-
-  free(*bbd_data);
-  *bbd_data = NULL;
-
-}
-
-int CVBBDPrecGetWorkSpace(void *bbd_data, long int *lenrwBBDP, long int *leniwBBDP)
+int CVBBDPrecGetWorkSpace(void *cvode_mem, long int *lenrwBBDP, long int *leniwBBDP)
 {
+  CVodeMem cv_mem;
+  CVSpilsMem cvspils_mem;
   CVBBDPrecData pdata;
 
-  if (bbd_data == NULL) {
-    CVProcessError(NULL, CVBBDPRE_PDATA_NULL, "CVBBDPRE", "CVBBDPrecGetWorkSpace", MSGBBDP_PDATA_NULL);    
-    return(CVBBDPRE_PDATA_NULL);
+  if (cvode_mem == NULL) {
+    CVProcessError(NULL, CVSPILS_MEM_NULL, "CVBBDPRE", "CVBBDPrecGetWorkSpace", MSGBBD_MEM_NULL);
+    return(CVSPILS_MEM_NULL);
+  }
+  cv_mem = (CVodeMem) cvode_mem;
+
+  if (cv_mem->cv_lmem == NULL) {
+    CVProcessError(cv_mem, CVSPILS_LMEM_NULL, "CVBBDPRE", "CVBBDPrecGetWorkSpace", MSGBBD_LMEM_NULL);
+    return(CVSPILS_LMEM_NULL);
+  }
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+
+  if (cvspils_mem->s_P_data == NULL) {
+    CVProcessError(cv_mem, CVSPILS_PMEM_NULL, "CVBBDPRE", "CVBBDPrecGetWorkSpace", MSGBBD_PMEM_NULL);
+    return(CVSPILS_PMEM_NULL);
   } 
-
-  pdata = (CVBBDPrecData) bbd_data;
+  pdata = (CVBBDPrecData) cvspils_mem->s_P_data;
 
   *lenrwBBDP = pdata->rpwsize;
   *leniwBBDP = pdata->ipwsize;
 
-  return(CVBBDPRE_SUCCESS);
+  return(CVSPILS_SUCCESS);
 }
 
-int CVBBDPrecGetNumGfnEvals(void *bbd_data, long int *ngevalsBBDP)
+int CVBBDPrecGetNumGfnEvals(void *cvode_mem, long int *ngevalsBBDP)
 {
+  CVodeMem cv_mem;
+  CVSpilsMem cvspils_mem;
   CVBBDPrecData pdata;
 
-  if (bbd_data == NULL) {
-    CVProcessError(NULL, CVBBDPRE_PDATA_NULL, "CVBBDPRE", "CVBBDPrecGetNumGfnEvals", MSGBBDP_PDATA_NULL);
-    return(CVBBDPRE_PDATA_NULL);
-  } 
+  if (cvode_mem == NULL) {
+    CVProcessError(NULL, CVSPILS_MEM_NULL, "CVBBDPRE", "CVBBDPrecGetNumGfnEvals", MSGBBD_MEM_NULL);
+    return(CVSPILS_MEM_NULL);
+  }
+  cv_mem = (CVodeMem) cvode_mem;
 
-  pdata = (CVBBDPrecData) bbd_data;
+  if (cv_mem->cv_lmem == NULL) {
+    CVProcessError(cv_mem, CVSPILS_LMEM_NULL, "CVBBDPRE", "CVBBDPrecGetNumGfnEvals", MSGBBD_LMEM_NULL);
+    return(CVSPILS_LMEM_NULL);
+  }
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+
+  if (cvspils_mem->s_P_data == NULL) {
+    CVProcessError(cv_mem, CVSPILS_PMEM_NULL, "CVBBDPRE", "CVBBDPrecGetNumGfnEvals", MSGBBD_PMEM_NULL);
+    return(CVSPILS_PMEM_NULL);
+  } 
+  pdata = (CVBBDPrecData) cvspils_mem->s_P_data;
 
   *ngevalsBBDP = pdata->nge;
 
-  return(CVBBDPRE_SUCCESS);
-}
-
-/*
- * -----------------------------------------------------------------
- * CVBBDPrecGetReturnFlagName
- * -----------------------------------------------------------------
- */
-
-char *CVBBDPrecGetReturnFlagName(int flag)
-{
-  char *name;
-
-  name = (char *)malloc(30*sizeof(char));
-
-  switch(flag) {
-  case CVBBDPRE_SUCCESS:
-    sprintf(name,"CVBBDPRE_SUCCESS");
-    break;   
-  case CVBBDPRE_PDATA_NULL:
-    sprintf(name,"CVBBDPRE_PDATA_NULL");
-    break;
-  case CVBBDPRE_FUNC_UNRECVR:
-    sprintf(name,"CVBBDPRE_FUNC_UNRECVR");
-    break;
-  default:
-    sprintf(name,"NONE");
-  }
-
-  return(name);
+  return(CVSPILS_SUCCESS);
 }
 
 /* Readability Replacements */
@@ -375,9 +328,8 @@ char *CVBBDPrecGetReturnFlagName(int flag)
  *
  * gamma   is the scalar appearing in the Newton matrix.
  *
- * bbd_data  is a pointer to user data - the same as the P_data
- *           parameter passed to CVSp*. For CVBBDPrecon,
- *           this should be of type CVBBDData.
+ * bbd_data is a pointer to the preconditioner data set by
+ *          CVBBDPrecInit
  *
  * tmp1, tmp2, and tmp3 are pointers to memory allocated
  *           for NVectors which are be used by CVBBDPrecSetup
@@ -395,7 +347,7 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
                           realtype gamma, void *bbd_data, 
                           N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
-  long int ier;
+  int ier;
   CVBBDPrecData pdata;
   CVodeMem cv_mem;
   int retval;
@@ -414,11 +366,11 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
 
     /* Otherwise call CVBBDDQJac for new J value */
     *jcurPtr = TRUE;
-    BandZero(savedJ);
+    SetToZero(savedJ);
 
     retval = CVBBDDQJac(pdata, t, y, tmp1, tmp2, tmp3);
     if (retval < 0) {
-      CVProcessError(cv_mem, CVBBDPRE_FUNC_UNRECVR, "CVBBDPRE", "CVBBDPrecSetup", MSGBBDP_FUNC_FAILED);
+      CVProcessError(cv_mem, -1, "CVBBDPRE", "CVBBDPrecSetup", MSGBBD_FUNC_FAILED);
       return(-1);
     }
     if (retval > 0) {
@@ -431,7 +383,7 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
   
   /* Scale and add I to get P = I - gamma*J */
   BandScale(-gamma, savedP);
-  BandAddI(savedP);
+  AddIdentity(savedP);
  
   /* Do LU factorization of P in place */
   ier = BandGBTRF(savedP, pivots);
@@ -451,12 +403,12 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
  *
  * The parameters of CVBBDPrecSolve used here are as follows:
  *
- * r      is the right-hand side vector of the linear system.
+ * r is the right-hand side vector of the linear system.
  *
- * bbd_data is a pointer to the preconditioner data returned by
- *          CVBBDPrecAlloc.
+ * bbd_data is a pointer to the preconditioner data set by
+ *   CVBBDPrecInit.
  *
- * z      is the output vector computed by CVBBDPrecSolve.
+ * z is the output vector computed by CVBBDPrecSolve.
  *
  * The value returned by the CVBBDPrecSolve function is always 0,
  * indicating success.
@@ -483,9 +435,30 @@ static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy,
   return(0);
 }
 
-#define ewt    (cv_mem->cv_ewt)
-#define h      (cv_mem->cv_h)
-#define f_data (cv_mem->cv_f_data)
+
+static void CVBBDPrecFree(CVodeMem cv_mem)
+{
+  CVSpilsMem cvspils_mem;
+  CVBBDPrecData pdata;
+  
+  if (cv_mem->cv_lmem == NULL) return;
+  cvspils_mem = (CVSpilsMem) cv_mem->cv_lmem;
+  
+  if (cvspils_mem->s_P_data == NULL) return;
+  pdata = (CVBBDPrecData) cvspils_mem->s_P_data;
+
+  DestroyMat(savedJ);
+  DestroyMat(savedP);
+  DestroyArray(pivots);
+
+  free(pdata);
+  pdata = NULL;
+}
+
+
+#define ewt       (cv_mem->cv_ewt)
+#define h         (cv_mem->cv_h)
+#define user_data (cv_mem->cv_user_data)
 
 /*
  * -----------------------------------------------------------------
@@ -493,7 +466,7 @@ static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy,
  * -----------------------------------------------------------------
  * This routine generates a banded difference quotient approximation
  * to the local block of the Jacobian of g(t,y). It assumes that a
- * band matrix of type BandMat is stored columnwise, and that elements
+ * band matrix of type DlsMat is stored columnwise, and that elements
  * within each column are contiguous. All matrix elements are generated
  * as difference quotients, by way of calls to the user routine gloc.
  * By virtue of the band structure, the number of these calls is
@@ -510,7 +483,7 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t,
 {
   CVodeMem cv_mem;
   realtype gnorm, minInc, inc, inc_inv;
-  long int group, i, j, width, ngroups, i1, i2;
+  int group, i, j, width, ngroups, i1, i2;
   realtype *y_data, *ewt_data, *gy_data, *gtemp_data, *ytemp_data, *col_j;
   int retval;
 
@@ -521,11 +494,11 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t,
 
   /* Call cfn and gloc to get base value of g(t,y) */
   if (cfn != NULL) {
-    retval = cfn(Nlocal, t, y, f_data);
+    retval = cfn(Nlocal, t, y, user_data);
     if (retval != 0) return(retval);
   }
 
-  retval = gloc(Nlocal, t, ytemp, gy, f_data);
+  retval = gloc(Nlocal, t, ytemp, gy, user_data);
   nge++;
   if (retval != 0) return(retval);
 
@@ -555,7 +528,7 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t,
     }
 
     /* Evaluate g with incremented y */
-    retval = gloc(Nlocal, t, ytemp, gtemp, f_data);
+    retval = gloc(Nlocal, t, ytemp, gtemp, user_data);
     nge++;
     if (retval != 0) return(retval);
 
