@@ -1,7 +1,7 @@
 /*
  * -----------------------------------------------------------------
- * $Revision: 1.1 $
- * $Date: 2006/07/05 15:32:36 $
+ * $Revision: 1.7 $
+ * $Date: 2009/03/29 23:28:01 $
  * -----------------------------------------------------------------
  * Programmer(s): Allan Taylor, Alan Hindmarsh, Radu Serban, and
  *                Aaron Collier @ LLNL
@@ -19,7 +19,7 @@
  * ------------------
  *   Creation and allocation functions
  *     KINCreate
- *     KINMalloc
+ *     KINInit
  *   Main solver function
  *     KINSol
  *   Deallocation function
@@ -195,7 +195,7 @@ static int KINStop(KINMem kin_mem, int strategy, booleantype maxStepTaken, int s
  * KINCreate creates an internal memory block for a problem to 
  * be solved by KINSOL. If successful, KINCreate returns a pointer
  * to the problem memory. This pointer should be passed to
- * KINMalloc. If an initialization error occurs, KINCreate prints
+ * KINInit. If an initialization error occurs, KINCreate prints
  * an error message to standard error and returns NULL. 
  */
 
@@ -218,16 +218,16 @@ void *KINCreate(void)
   /* set default values for solver optional inputs */
 
   kin_mem->kin_func             = NULL;
-  kin_mem->kin_f_data           = NULL;
+  kin_mem->kin_user_data        = NULL;
   kin_mem->kin_constraints      = NULL;
   kin_mem->kin_uscale           = NULL;
   kin_mem->kin_fscale           = NULL;
   kin_mem->kin_constraintsSet   = FALSE;
   kin_mem->kin_ehfun            = KINErrHandler;
-  kin_mem->kin_eh_data          = (void *) kin_mem;
+  kin_mem->kin_eh_data          = kin_mem;
   kin_mem->kin_errfp            = stderr;
   kin_mem->kin_ihfun            = KINInfoHandler;
-  kin_mem->kin_ih_data          = (void *) kin_mem;
+  kin_mem->kin_ih_data          = kin_mem;
   kin_mem->kin_infofp           = stdout;
   kin_mem->kin_printfl          = PRINTFL_DEFAULT;
   kin_mem->kin_mxiter           = MXITER_DEFAULT;
@@ -249,7 +249,8 @@ void *KINCreate(void)
   kin_mem->kin_eta_gamma        = POINT9;     /* default for KIN_ETACHOICE2  */
   kin_mem->kin_MallocDone       = FALSE;
   kin_mem->kin_setupNonNull     = FALSE;
-  kin_mem->kin_omega            = ZERO;       /* default to using min / max  */
+  kin_mem->kin_eval_omega       = TRUE;
+  kin_mem->kin_omega            = ZERO;       /* default to using min/max    */
   kin_mem->kin_omega_min        = OMEGA_MIN;
   kin_mem->kin_omega_max        = OMEGA_MAX;
 
@@ -258,7 +259,7 @@ void *KINCreate(void)
   kin_mem->kin_lrw = 17;
   kin_mem->kin_liw = 22;
 
-  /* NOTE: needed since KINMalloc could be called after KINSetConstraints */
+  /* NOTE: needed since KINInit could be called after KINSetConstraints */
 
   kin_mem->kin_lrw1 = 0;
   kin_mem->kin_liw1 = 0;
@@ -271,15 +272,15 @@ void *KINCreate(void)
 #define lrw   (kin_mem->kin_lrw)
 
 /*
- * Function : KINMalloc
+ * Function : KINInit
  *
- * KINMalloc allocates memory for a problem or execution of KINSol. 
+ * KINInit allocates memory for a problem or execution of KINSol. 
  * If memory is successfully allocated, KIN_SUCCESS is returned.
  * Otherwise, an error message is printed and an error flag
  * returned.
  */
 
-int KINMalloc(void *kinmem, KINSysFn func, N_Vector tmpl)
+int KINInit(void *kinmem, KINSysFn func, N_Vector tmpl)
 {
   long int liw1, lrw1;
   KINMem kin_mem;
@@ -288,13 +289,13 @@ int KINMalloc(void *kinmem, KINSysFn func, N_Vector tmpl)
   /* check kinmem */
 
   if (kinmem == NULL) {
-    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINMalloc", MSG_NO_MEM);
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINInit", MSG_NO_MEM);
     return(KIN_MEM_NULL);
   }
   kin_mem = (KINMem) kinmem;
 
   if (func == NULL) {
-    KINProcessError(kin_mem, KIN_ILL_INPUT, "KINSOL", "KINMalloc", MSG_FUNC_NULL);
+    KINProcessError(kin_mem, KIN_ILL_INPUT, "KINSOL", "KINInit", MSG_FUNC_NULL);
     return(KIN_ILL_INPUT);
   }
 
@@ -302,7 +303,7 @@ int KINMalloc(void *kinmem, KINSysFn func, N_Vector tmpl)
 
   nvectorOK = KINCheckNvector(tmpl);
   if (!nvectorOK) {
-    KINProcessError(kin_mem, KIN_ILL_INPUT, "KINSOL", "KINMalloc", MSG_BAD_NVECTOR);
+    KINProcessError(kin_mem, KIN_ILL_INPUT, "KINSOL", "KINInit", MSG_BAD_NVECTOR);
     return(KIN_ILL_INPUT);
   }
 
@@ -322,7 +323,7 @@ int KINMalloc(void *kinmem, KINSysFn func, N_Vector tmpl)
 
   allocOK = KINAllocVectors(kin_mem, tmpl);
   if (!allocOK) {
-    KINProcessError(kin_mem, KIN_MEM_FAIL, "KINSOL", "KINMalloc", MSG_MEM_FAIL);
+    KINProcessError(kin_mem, KIN_MEM_FAIL, "KINSOL", "KINInit", MSG_MEM_FAIL);
     free(kin_mem); kin_mem = NULL;
     return(KIN_MEM_FAIL);
   }
@@ -353,7 +354,7 @@ int KINMalloc(void *kinmem, KINSysFn func, N_Vector tmpl)
  */
 
 #define func             (kin_mem->kin_func)
-#define f_data           (kin_mem->kin_f_data)
+#define user_data        (kin_mem->kin_user_data)
 #define printfl          (kin_mem->kin_printfl)
 #define mxiter           (kin_mem->kin_mxiter)
 #define noInitSetup      (kin_mem->kin_noInitSetup)
@@ -414,6 +415,7 @@ int KINMalloc(void *kinmem, KINSysFn func, N_Vector tmpl)
 #define msbset_sub       (kin_mem->kin_msbset_sub)
 #define nnilset_sub      (kin_mem->kin_nnilset_sub)
 #define update_fnorm_sub (kin_mem->kin_update_fnorm_sub)
+#define eval_omega       (kin_mem->kin_eval_omega)
 #define omega            (kin_mem->kin_omega)
 #define omega_min        (kin_mem->kin_omega_min)
 #define omega_max        (kin_mem->kin_omega_max)
@@ -500,6 +502,13 @@ int KINSol(void *kinmem, N_Vector u, int strategy,
 
   if (inexact_ls && !noMinEps) epsmin = POINT01 * fnormtol;
 
+
+  /* if omega is zero at this point, make sure it will be evaluated
+     at each iteration based on the provided min/max bounds and the
+     current function norm. */
+  if (omega == ZERO) eval_omega = TRUE;
+  else               eval_omega = FALSE;
+ 
   loop{
 
     retry_nni = FALSE;
@@ -632,7 +641,7 @@ int KINSol(void *kinmem, N_Vector u, int strategy,
 /*
  * Function : KINFree
  *
- * This routine frees the problem memory allocated by KINMalloc.
+ * This routine frees the problem memory allocated by KINInit.
  * Such memory includes all the vectors allocated by
  * KINAllocVectors, and the memory lmem for the linear solver
  * (deallocated by a call to lfree).
@@ -705,18 +714,15 @@ static booleantype KINAllocVectors(KINMem kin_mem, N_Vector tmpl)
 {
   /* allocate unew, fval, pp, vtemp1 and vtemp2 */
   
-  unew = NULL;
   unew = N_VClone(tmpl);
   if (unew == NULL) return(FALSE);
 
-  fval = NULL;
   fval = N_VClone(tmpl);
   if (fval == NULL) {
     N_VDestroy(unew);
     return(FALSE);
   }
 
-  pp = NULL;
   pp = N_VClone(tmpl);
   if (pp == NULL) {
     N_VDestroy(unew);
@@ -724,7 +730,6 @@ static booleantype KINAllocVectors(KINMem kin_mem, N_Vector tmpl)
     return(FALSE);
   }
 
-  vtemp1 = NULL;
   vtemp1 = N_VClone(tmpl);
   if (vtemp1 == NULL) {
     N_VDestroy(unew);
@@ -733,7 +738,6 @@ static booleantype KINAllocVectors(KINMem kin_mem, N_Vector tmpl)
     return(FALSE);
   }
 
-  vtemp2 = NULL;
   vtemp2 = N_VClone(tmpl);
   if (vtemp2 == NULL) {
     N_VDestroy(unew);
@@ -906,7 +910,7 @@ static int KINSolInit(KINMem kin_mem, int strategy)
 
   /* see if the system func(uu) = 0 is satisfied by the initial guess uu */
 
-  retval = func(uu, fval, f_data); nfe++;
+  retval = func(uu, fval, user_data); nfe++;
   if (retval < 0) {
     KINProcessError(kin_mem, KIN_SYSFUNC_FAIL, "KINSOL", "KINSolInit", MSG_SYSFUNC_FAILED);
     return(KIN_SYSFUNC_FAIL);
@@ -1066,7 +1070,7 @@ static int KINFullNewton(KINMem kin_mem, realtype *fnormp, realtype *f1normp,
     N_VLinearSum(ONE, uu, ONE, pp, unew);
 
     /* evaluate func(unew) and its norm, and return */
-    retval = func(unew, fval, f_data); nfe++;
+    retval = func(unew, fval, user_data); nfe++;
 
     /* if func was successful, accept pp */
     if (retval == 0) {fOK = TRUE; break;}
@@ -1215,7 +1219,7 @@ static int KINLineSearch(KINMem kin_mem, realtype *fnormp, realtype *f1normp,
     N_VLinearSum(ONE, uu, ONE, pp, unew);
 
     /* evaluate func(unew) and its norm, and return */
-    retval = func(unew, fval, f_data); nfe++;
+    retval = func(unew, fval, user_data); nfe++;
 
     /* if func was successful, accept pp */
     if (retval == 0) {fOK = TRUE; break;}
@@ -1307,7 +1311,7 @@ static int KINLineSearch(KINMem kin_mem, realtype *fnormp, realtype *f1normp,
 
     N_VLinearSum(ONE, uu, rl, pp, unew);
 
-    retval = func(unew, fval, f_data); nfe++;
+    retval = func(unew, fval, user_data); nfe++;
     if (retval != 0) return(KIN_SYSFUNC_FAIL);
 
     *fnormp = N_VWL2Norm(fval, fscale);
@@ -1344,7 +1348,7 @@ static int KINLineSearch(KINMem kin_mem, realtype *fnormp, realtype *f1normp,
         nbktrk_l++;
 
         N_VLinearSum(ONE, uu, rl, pp, unew);
-        retval = func(unew, fval, f_data); nfe++;
+        retval = func(unew, fval, user_data); nfe++;
         if (retval != 0) return(KIN_SYSFUNC_FAIL);
         *fnormp = N_VWL2Norm(fval, fscale);
         *f1normp = HALF * (*fnormp) * (*fnormp);
@@ -1373,7 +1377,7 @@ static int KINLineSearch(KINMem kin_mem, realtype *fnormp, realtype *f1normp,
         nbktrk_l++;
 
         N_VLinearSum(ONE, uu, rl, pp, unew);
-        retval = func(unew, fval, f_data); nfe++;
+        retval = func(unew, fval, user_data); nfe++;
         if (retval != 0) return(KIN_SYSFUNC_FAIL);
         *fnormp = N_VWL2Norm(fval, fscale);
         *f1normp = HALF * (*fnormp) * (*fnormp);
@@ -1400,7 +1404,7 @@ static int KINLineSearch(KINMem kin_mem, realtype *fnormp, realtype *f1normp,
 	   that satisfied the alpha condition and continue */
 
         N_VLinearSum(ONE, uu, rllo, pp, unew);
-        retval = func(unew, fval, f_data); nfe++;
+        retval = func(unew, fval, user_data); nfe++;
         if (retval != 0) return(KIN_SYSFUNC_FAIL);
         *fnormp = N_VWL2Norm(fval, fscale);
         *f1normp = HALF * (*fnormp) * (*fnormp);   
@@ -1565,8 +1569,8 @@ static int KINStop(KINMem kin_mem, int strategy, booleantype maxStepTaken, int s
 
       nnilset_sub = nni;
 
-      /* If OMEGA is still zero at this point, estimate it */
-      if (omega == ZERO)
+      /* If indicated, estimate new OMEGA value */
+      if (eval_omega)
         omega = MIN(omega_min*EXP(MAX(ZERO,(fnorm/fnormtol)-ONE)), omega_max);
 
       /* Check if making satisfactory progress */
@@ -1818,8 +1822,10 @@ void KINInfoHandler(const char *module, const char *function,
   kin_mem = (KINMem) data;
   
 #ifndef NO_FPRINTF_OUTPUT
-  fprintf(infofp,"\n[%s] %s\n",module, function);
-  fprintf(infofp,"   %s\n",msg);
+  if (infofp != NULL) {
+    fprintf(infofp,"\n[%s] %s\n",module, function);
+    fprintf(infofp,"   %s\n",msg);
+  }
 #endif  
 
 }
@@ -1907,7 +1913,7 @@ void KINErrHandler(int error_code, const char *module,
     sprintf(err_type,"ERROR");
 
 #ifndef NO_FPRINTF_OUTPUT
-  if (errfp!=NULL) {
+  if (errfp != NULL) {
     fprintf(errfp,"\n[%s %s]  %s\n",module,err_type,function);
     fprintf(errfp,"  %s\n\n",msg);
   }
